@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import json
 import os
 import shutil
@@ -6,10 +8,11 @@ from contextlib import suppress
 from datetime import datetime
 from pathlib import Path
 from time import time
-from typing import Any, Dict, Optional, Union
+from typing import Any
 
 from streamlink.compat import is_win32
 
+
 if is_win32:
     xdg_cache = os.environ.get("APPDATA", os.path.expanduser("~"))
 else:
@@ -25,13 +28,22 @@ CACHE_DIR = Path(xdg_cache) / "streamlink"
 #  - add JSON schema information
 #  - add translation logic, to keep backwards compatibility
 class Cache:
-    """Caches Python values as JSON and prunes expired entries."""
+    def __init__(
+        self,
+        filename: str | Path,
+        key_prefix: str = "",
+    ):
+        """
+        Caches Python values as JSON and prunes expired entries.
+
+        :param filename: A file name or :class:`Path` object, relative to the cache directory
+        :param key_prefix: Optional prefix for each key to be retrieved from or stored in the cache
+        """
 
-    def __init__(self, filename: Union[str, Path], key_prefix: str = ""):
         self.key_prefix = key_prefix
         self.filename = CACHE_DIR / Path(filename)
 
-        self._cache: Dict[str, Dict[str, Any]] = {}
+        self._cache: dict[str, dict[str, Any]] = {}
 
     def _load(self):
         self._cache = {}
@@ -73,7 +85,24 @@ class Cache:
             with suppress(Exception):
                 os.remove(tempname)
 
-    def set(self, key: str, value: Any, expires: float = 60 * 60 * 24 * 7, expires_at: Optional[datetime] = None):
+    def set(
+        self,
+        key: str,
+        value: Any,
+        expires: float = 60 * 60 * 24 * 7,
+        expires_at: datetime | None = None,
+    ) -> None:
+        """
+        Store the given value using the key name and expiration time.
+
+        Prunes the cache of all expired key-value pairs before setting the new key-value pair.
+
+        :param key: A specific key name
+        :param value: Any kind of value that can be JSON-serialized
+        :param expires: Expiration time in seconds, with the default being one week
+        :param expires_at: Optional expiration date, which overrides the expiration time
+        """
+
         self._load()
         self._prune()
 
@@ -91,7 +120,21 @@ class Cache:
         self._cache[key] = dict(value=value, expires=expires)
         self._save()
 
-    def get(self, key: str, default: Optional[Any] = None):
+    def get(
+        self,
+        key: str,
+        default: Any | None = None,
+    ) -> Any:
+        """
+        Attempt to retrieve the given key from the cache.
+
+        Prunes the cache of all expired key-value pairs before retrieving the key's value.
+
+        :param key: A specific key name
+        :param default: An optional default value if no key was stored, or if it has expired
+        :return: The retrieved value or optional default value
+        """
+
         self._load()
 
         if self._prune():
@@ -105,7 +148,15 @@ class Cache:
         else:
             return default
 
-    def get_all(self):
+    def get_all(self) -> dict[str, Any]:
+        """
+        Retrieve all cached key-value pairs.
+
+        Prunes the cache of all expired key-value pairs first.
+
+        :return: A dictionary of all cached key-value pairs.
+        """
+
         ret = {}
         self._load()
 
@@ -118,7 +169,7 @@ class Cache:
             else:
                 prefix = ""
             if key.startswith(prefix):
-                okey = key[len(prefix):]
+                okey = key[len(prefix) :]
                 ret[okey] = value["value"]
 
         return ret
