@@ -5,14 +5,17 @@ $url video.ibm.com
 $type live, vod
 """
 
+from __future__ import annotations
+
 import logging
 import re
 from collections import deque
+from collections.abc import Callable, Mapping
 from dataclasses import dataclass
 from datetime import datetime, timedelta, timezone
 from random import randint
 from threading import Event, RLock
-from typing import Any, Callable, Deque, Dict, List, Optional, Union
+from typing import Any
 from urllib.parse import urljoin, urlunparse
 
 from requests import Response
@@ -56,7 +59,7 @@ class UStreamTVSegment(Segment):
     path: str
 
     # the segment URLs depend on the CDN and the chosen stream format and its segment template string
-    def url(self, base: Optional[str], template: str) -> str:
+    def url(self, base: str | None, template: str) -> str:
         return urljoin(
             base or "",
             f"{self.path}/{template.replace('%', str(self.num), 1).replace('%', self.hash, 1)}",
@@ -75,52 +78,56 @@ class UStreamTVWsClient(WebsocketClient):
         "args": [{str: object}],
     })
     _schema_stream_formats = validate.Schema({
-        "streams": [validate.any(
-            validate.all(
-                {
-                    "contentType": "video/mp4",
-                    "sourceStreamVersion": int,
-                    "initUrl": str,
-                    "segmentUrl": str,
-                    "bitrate": int,
-                    "height": int,
-                },
-                validate.transform(lambda obj: StreamFormatVideo(**obj)),
+        "streams": [
+            validate.any(
+                validate.all(
+                    {
+                        "contentType": "video/mp4",
+                        "sourceStreamVersion": int,
+                        "initUrl": str,
+                        "segmentUrl": str,
+                        "bitrate": int,
+                        "height": int,
+                    },
+                    validate.transform(lambda obj: StreamFormatVideo(**obj)),
+                ),
+                validate.all(
+                    {
+                        "contentType": "audio/mp4",
+                        "sourceStreamVersion": int,
+                        "initUrl": str,
+                        "segmentUrl": str,
+                        "bitrate": int,
+                        validate.optional("language"): str,
+                    },
+                    validate.transform(lambda obj: StreamFormatAudio(**obj)),
+                ),
+                object,
             ),
-            validate.all(
-                {
-                    "contentType": "audio/mp4",
-                    "sourceStreamVersion": int,
-                    "initUrl": str,
-                    "segmentUrl": str,
-                    "bitrate": int,
-                    validate.optional("language"): str,
-                },
-                validate.transform(lambda obj: StreamFormatAudio(**obj)),
-            ),
-            object,
-        )],
+        ],
     })
     _schema_stream_segments = validate.Schema({
         "chunkId": int,
         "chunkTime": int,
         "contentAccess": validate.all(
             {
-                "accessList": [{
-                    "data": {
-                        "path": str,
+                "accessList": [
+                    {
+                        "data": {
+                            "path": str,
+                        },
                     },
-                }],
+                ],
             },
             validate.get(("accessList", 0, "data", "path")),
         ),
         "hashes": {validate.transform(int): str},
     })
 
-    stream_cdn: Optional[str] = None
-    stream_formats_video: Optional[List[StreamFormatVideo]] = None
-    stream_formats_audio: Optional[List[StreamFormatAudio]] = None
-    stream_initial_id: Optional[int] = None
+    stream_cdn: str | None = None
+    stream_formats_video: list[StreamFormatVideo] | None = None
+    stream_formats_audio: list[StreamFormatAudio] | None = None
+    stream_initial_id: int | None = None
 
     def __init__(
         self,
@@ -137,8 +144,8 @@ class UStreamTVWsClient(WebsocketClient):
         self.ready = Event()
         self.stream_error = None
         # a list of deques subscribed by worker threads which independently need to read segments
-        self.stream_segments_subscribers: List[Deque[UStreamTVSegment]] = []
-        self.stream_segments_initial: Deque[UStreamTVSegment] = deque()
+        self.stream_segments_subscribers: list[deque[UStreamTVSegment]] = []
+        self.stream_segments_initial: deque[UStreamTVSegment] = deque()
         self.stream_segments_lock = RLock()
 
         self.media_id = media_id
@@ -152,7 +159,7 @@ class UStreamTVWsClient(WebsocketClient):
         super().__init__(session, self._get_url(), origin="https://www.ustream.tv")
 
     def _get_url(self):
-        return self.API_URL.format(randint(0, 0xffffff), self.media_id, self.application, self.cluster)
+        return self.API_URL.format(randint(0, 0xFFFFFF), self.media_id, self.application, self.cluster)
 
     def _set_error(self, error: Any):
         self.stream_error = error
@@ -168,7 +175,7 @@ class UStreamTVWsClient(WebsocketClient):
                 log.info("Closing websocket connection")
                 self.ws.close()
 
-    def segments_subscribe(self) -> Deque[UStreamTVSegment]:
+    def segments_subscribe(self) -> deque[UStreamTVSegment]:
         with self.stream_segments_lock:
             # copy the initial segments deque (segments arrive early)
             new_deque = self.stream_segments_initial.copy()
@@ -212,7 +219,7 @@ class UStreamTVWsClient(WebsocketClient):
             return
 
         cmd: str = parsed["cmd"]
-        args: List[Dict] = parsed["args"]
+        args: list[dict] = parsed["args"]
         log.trace(f"Received '{cmd}' command")  # type: ignore[attr-defined]
         log.trace(f"{args!r}")  # type: ignore[attr-defined]
 
@@ -226,7 +233,7 @@ class UStreamTVWsClient(WebsocketClient):
                         handler(self, argdata)
 
     # noinspection PyMethodMayBeStatic
-    def _handle_warning(self, data: Dict):
+    def _handle_warning(self, data: dict):
         log.warning(f"{data['code']}: {str(data['message'])[:50]}")
 
     # noinspection PyUnusedLocal
@@ -237,26 +244,28 @@ class UStreamTVWsClient(WebsocketClient):
     def _handle_reject_geo_lock(self, *args):
         self._set_error("This content is not available in your area")
 
-    def _handle_reject_cluster(self, arg: Dict):
+    def _handle_reject_cluster(self, arg: dict):
         self.cluster = arg["name"]
         log.info(f"Switching cluster to: {self.cluster}")
         self.reconnect(url=self._get_url())
 
-    def _handle_reject_referrer_lock(self, arg: Dict):
+    def _handle_reject_referrer_lock(self, arg: dict):
         self.referrer = arg["redirectUrl"]
         log.info(f"Updating referrer to: {self.referrer}")
         self.reconnect(url=self._get_url())
 
-    def _handle_module_info_cdn_config(self, data: Dict):
+    def _handle_module_info_cdn_config(self, data: dict):
         self.stream_cdn = urlunparse((
             data["protocol"],
             data["data"][0]["data"][0]["sites"][0]["host"],
             data["data"][0]["data"][0]["sites"][0]["path"],
-            "", "", "",
+            "",
+            "",
+            "",
         ))
         self._set_ready()
 
-    def _handle_module_info_stream(self, data: Dict):
+    def _handle_module_info_stream(self, data: dict):
         if data.get("contentAvailable") is False:
             return self._set_error("This stream is currently offline")
 
@@ -276,7 +285,7 @@ class UStreamTVWsClient(WebsocketClient):
 
         # parse segment duration and hashes, and queue new segments
         try:
-            segmentdata: Dict = self._schema_stream_segments.validate(mp4_segmented)
+            segmentdata: dict = self._schema_stream_segments.validate(mp4_segmented)
         except PluginError:
             log.error("Failed parsing hashes")
             return
@@ -284,7 +293,7 @@ class UStreamTVWsClient(WebsocketClient):
         current_id: int = segmentdata["chunkId"]
         duration: int = segmentdata["chunkTime"]
         path: str = segmentdata["contentAccess"]
-        hashes: Dict[int, str] = segmentdata["hashes"]
+        hashes: dict[int, str] = segmentdata["hashes"]
 
         sorted_ids = sorted(hashes.keys())
         count = len(sorted_ids)
@@ -308,20 +317,22 @@ class UStreamTVWsClient(WebsocketClient):
                     # the last id->hash item will use the previous diff to extrapolate segment IDs
                     diff = sorted_ids[idx_next] - segment_id
                 for num in range(segment_id, segment_id + diff):
-                    self._segments_append(UStreamTVSegment(
-                        uri="",
-                        num=num,
-                        duration=duration,
-                        available_at=current_time + timedelta(seconds=(num - current_id - 1) * duration / 1000),
-                        hash=hashes[segment_id],
-                        path=path,
-                    ))
+                    self._segments_append(
+                        UStreamTVSegment(
+                            uri="",
+                            num=num,
+                            duration=duration,
+                            available_at=current_time + timedelta(seconds=(num - current_id - 1) * duration / 1000),
+                            hash=hashes[segment_id],
+                            path=path,
+                        ),
+                    )
 
         self._set_ready()
 
     # ----
 
-    _MESSAGE_HANDLERS: Dict[str, Dict[str, Callable[["UStreamTVWsClient", Any], None]]] = {
+    _MESSAGE_HANDLERS: Mapping[str, Mapping[str, Callable[[UStreamTVWsClient, Any], None]]] = {
         "warning": {
             "code": _handle_warning,
         },
@@ -339,8 +350,8 @@ class UStreamTVWsClient(WebsocketClient):
 
 
 class UStreamTVStreamWriter(SegmentedStreamWriter[UStreamTVSegment, Response]):
-    reader: "UStreamTVStreamReader"
-    stream: "UStreamTVStream"
+    reader: UStreamTVStreamReader
+    stream: UStreamTVStream
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -395,9 +406,9 @@ class UStreamTVStreamWriter(SegmentedStreamWriter[UStreamTVSegment, Response]):
 
 
 class UStreamTVStreamWorker(SegmentedStreamWorker[UStreamTVSegment, Response]):
-    reader: "UStreamTVStreamReader"
-    writer: "UStreamTVStreamWriter"
-    stream: "UStreamTVStream"
+    reader: UStreamTVStreamReader
+    writer: UStreamTVStreamWriter
+    stream: UStreamTVStream
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -432,9 +443,9 @@ class UStreamTVStreamReader(SegmentedStreamReader[UStreamTVSegment, Response]):
     __worker__ = UStreamTVStreamWorker
     __writer__ = UStreamTVStreamWriter
 
-    stream: "UStreamTVStream"
-    worker: "UStreamTVStreamWorker"
-    writer: "UStreamTVStreamWriter"
+    stream: UStreamTVStream
+    worker: UStreamTVStreamWorker
+    writer: UStreamTVStreamWriter
 
     def open(self):
         self.stream.wsclient.opened.set()
@@ -453,7 +464,7 @@ class UStreamTVStream(Stream):
         session,
         kind: str,
         wsclient: UStreamTVWsClient,
-        stream_format: Union[StreamFormatVideo, StreamFormatAudio],
+        stream_format: StreamFormatVideo | StreamFormatAudio,
     ):
         super().__init__(session)
         self.kind = kind
@@ -467,17 +478,22 @@ class UStreamTVStream(Stream):
         return reader
 
 
-@pluginmatcher(re.compile(r"""
-    https?://(?:(?:www\.)?ustream\.tv|video\.ibm\.com)
-    (?:
-        /combined-embed
-        /(?P<combined_channel_id>\d+)
-        (?:/video/(?P<combined_video_id>\d+))?
-        |
-        (?:(?:/embed/|/channel/(?:id/)?)(?P<channel_id>\d+))?
-        (?:(?:/embed)?/recorded/(?P<video_id>\d+))?
-    )
-""", re.VERBOSE))
+@pluginmatcher(
+    re.compile(
+        r"""
+            https?://(?:(?:www\.)?ustream\.tv|video\.ibm\.com)
+            (?:
+                /combined-embed
+                /(?P<combined_channel_id>\d+)
+                (?:/video/(?P<combined_video_id>\d+))?
+                |
+                (?:(?:/embed/|/channel/(?:id/)?)(?P<channel_id>\d+))?
+                (?:(?:/embed)?/recorded/(?P<video_id>\d+))?
+            )
+        """,
+        re.VERBOSE,
+    ),
+)
 @pluginargument(
     "password",
     sensitive=True,
@@ -523,7 +539,8 @@ class UStreamTV(Plugin):
             password=self.get_option("password"),
         )
         log.debug(
-            "Connecting to UStream API: " + ", ".join([
+            "Connecting to UStream API: "
+            + ", ".join([
                 f"media_id={media_id}",
                 f"application={application}",
                 f"referrer={self.url}",
@@ -537,7 +554,7 @@ class UStreamTV(Plugin):
             not wsclient.ready.wait(self.STREAM_READY_TIMEOUT)
             or not wsclient.is_alive()
             or wsclient.stream_error
-        ):
+        ):  # fmt: skip
             log.error(wsclient.stream_error or "Waiting for stream data timed out.")
             wsclient.close()
             return
@@ -548,10 +565,13 @@ class UStreamTV(Plugin):
         else:
             for video in wsclient.stream_formats_video:
                 for audio in wsclient.stream_formats_audio:
-                    yield f"{video.height}p+a{audio.bitrate}k", MuxedStream(
-                        self.session,
-                        UStreamTVStream(self.session, "video", wsclient, video),
-                        UStreamTVStream(self.session, "audio", wsclient, audio),
+                    yield (
+                        f"{video.height}p+a{audio.bitrate}k",
+                        MuxedStream(
+                            self.session,
+                            UStreamTVStream(self.session, "video", wsclient, video),
+                            UStreamTVStream(self.session, "audio", wsclient, audio),
+                        ),
                     )
 
 
