@@ -1,6 +1,9 @@
+from __future__ import annotations
+
 import re
+from inspect import currentframe, getframeinfo
+from operator import itemgetter
 from socket import AF_INET, AF_INET6
-from typing import Dict
 from unittest.mock import Mock
 
 import pytest
@@ -10,6 +13,7 @@ from requests.adapters import HTTPAdapter
 from streamlink.exceptions import StreamlinkDeprecationWarning
 from streamlink.session import Streamlink
 from streamlink.session.http import TLSNoDHAdapter
+from streamlink.session.options import StreamlinkOptions
 
 
 _original_allowed_gai_family = urllib3.util.connection.allowed_gai_family  # type: ignore[attr-defined]
@@ -50,6 +54,31 @@ def test_session_wrapper_methods(session: Streamlink):
     assert session.get_option("non_existing") is None
 
 
+def test_session_option_set_deprecated(recwarn: pytest.WarningsRecorder, session: Streamlink):
+    def get_lineno():
+        frame = currentframe()
+        assert frame
+        assert frame.f_back
+        return getframeinfo(frame.f_back).lineno
+
+    class FakeStreamlinkOptions(StreamlinkOptions):
+        _MAP_SETTERS = {
+            "deprecated": StreamlinkOptions._factory_set_deprecated("new", int),
+        }
+
+    session.options = FakeStreamlinkOptions(session)
+    assert session.get_option("new") is None
+    assert recwarn.list == []
+
+    session.set_option("deprecated", 123)
+    lineno = get_lineno() - 1
+
+    assert session.get_option("new") == 123
+    assert [(item.filename, item.lineno, item.category, str(item.message)) for item in recwarn.list] == [
+        (__file__, lineno, StreamlinkDeprecationWarning, "`deprecated` has been deprecated in favor of the `new` option"),
+    ]
+
+
 def test_options_locale(monkeypatch: pytest.MonkeyPatch, session: Streamlink):
     monkeypatch.setattr("locale.getlocale", lambda: ("C", None))
     assert session.get_option("locale") is None
@@ -75,30 +104,31 @@ def test_options_locale(monkeypatch: pytest.MonkeyPatch, session: Streamlink):
 
 
 class TestOptionsInterface:
-    @pytest.fixture()
-    def adapters(self, monkeypatch: pytest.MonkeyPatch):
-        adapters = {
-            scheme: Mock(poolmanager=Mock(connection_pool_kw={}))
-            for scheme in ("http://", "https://", "foo://")
-        }
-        monkeypatch.setattr("streamlink.session.session.HTTPSession", Mock(return_value=Mock(adapters=adapters)))
+    def test_options_interface(self, session: Streamlink):
+        session.http.mount("custom://", TLSNoDHAdapter())
 
-        return adapters
+        a_http, a_https, a_custom, a_file = itemgetter("http://", "https://", "custom://", "file://")(session.http.adapters)
+        assert isinstance(a_http, HTTPAdapter)
+        assert isinstance(a_https, HTTPAdapter)
+        assert isinstance(a_custom, HTTPAdapter)
+        assert not isinstance(a_file, HTTPAdapter)
 
-    def test_options_interface(self, adapters: Dict[str, Mock], session: Streamlink):
         assert session.get_option("interface") is None
+        assert a_http.poolmanager.connection_pool_kw.get("source_address") is None
+        assert a_https.poolmanager.connection_pool_kw.get("source_address") is None
+        assert a_custom.poolmanager.connection_pool_kw.get("source_address") is None
 
         session.set_option("interface", "my-interface")
-        assert adapters["http://"].poolmanager.connection_pool_kw == {"source_address": ("my-interface", 0)}
-        assert adapters["https://"].poolmanager.connection_pool_kw == {"source_address": ("my-interface", 0)}
-        assert adapters["foo://"].poolmanager.connection_pool_kw == {}
         assert session.get_option("interface") == "my-interface"
+        assert a_http.poolmanager.connection_pool_kw.get("source_address") == ("my-interface", 0)
+        assert a_https.poolmanager.connection_pool_kw.get("source_address") == ("my-interface", 0)
+        assert a_custom.poolmanager.connection_pool_kw.get("source_address") == ("my-interface", 0)
 
         session.set_option("interface", None)
-        assert adapters["http://"].poolmanager.connection_pool_kw == {}
-        assert adapters["https://"].poolmanager.connection_pool_kw == {}
-        assert adapters["foo://"].poolmanager.connection_pool_kw == {}
         assert session.get_option("interface") is None
+        assert a_http.poolmanager.connection_pool_kw.get("source_address") is None
+        assert a_https.poolmanager.connection_pool_kw.get("source_address") is None
+        assert a_custom.poolmanager.connection_pool_kw.get("source_address") is None
 
         # doesn't raise
         session.set_option("interface", None)
