@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import itertools
+import logging
 import os
 import unittest
 from datetime import datetime, timedelta, timezone
@@ -936,6 +937,38 @@ class TestHLSStreamEncrypted(TestMixinStreamHLS, unittest.TestCase):
         assert data == self.content([segments[1]], prop="content_plain")
         assert mock_log.error.mock_calls == [call("Error while decrypting segment 0: PKCS#7 padding is incorrect.")]
 
+    def test_hls_encrypted_switch_methods(self):
+        aesKey1, aesIv1, key_aes128_1 = self.gen_key()
+        aesKey2, aesIv2, key_aes128_2 = self.gen_key()
+        key_none = Tag("EXT-X-KEY", {"METHOD": "NONE"})
+
+        segments = self.subject([
+            Playlist(
+                0,
+                [
+                    key_aes128_1,
+                    SegmentEnc(0, aesKey1, aesIv1),
+                    SegmentEnc(1, aesKey1, aesIv1),
+                    key_none,
+                    Segment(2),
+                    Segment(3),
+                    key_aes128_2,
+                    SegmentEnc(4, aesKey2, aesIv2),
+                    SegmentEnc(5, aesKey2, aesIv2),
+                ],
+                end=True,
+            ),
+        ])
+
+        self.await_write(6)
+        data = self.await_read(read_all=True)
+        self.await_close()
+
+        expected = self.content(segments, prop="content_plain", cond=lambda s: 0 <= s.num <= 1)
+        expected += self.content(segments, cond=lambda s: 2 <= s.num <= 3)
+        expected += self.content(segments, prop="content_plain", cond=lambda s: 4 <= s.num <= 5)
+        assert data == expected, "Switches between encryption key methods"
+
 
 @patch("streamlink.stream.hls.hls.HLSStreamWorker.wait", Mock(return_value=True))
 class TestHlsPlaylistReloadTime(TestMixinStreamHLS, unittest.TestCase):
@@ -1097,8 +1130,9 @@ class TestHlsExtAudio:
         monkeypatch.setattr("streamlink.stream.hls.hls.FFMPEGMuxer.is_usable", Mock(return_value=True))
 
     @pytest.fixture(autouse=True)
-    def _playlist(self, requests_mock: rm.Mocker):
-        with text("hls/test_2.m3u8") as playlist:
+    def _playlist(self, request: pytest.FixtureRequest, requests_mock: rm.Mocker):
+        params = getattr(request, "param", {})
+        with text(params.get("playlist", "hls/test_media_language.m3u8")) as playlist:
             requests_mock.get("http://mocked/path/master.m3u8", text=playlist.read())
             yield
 
@@ -1116,8 +1150,14 @@ class TestHlsExtAudio:
     @pytest.mark.parametrize(
         ("session", "selection"),
         [
-            pytest.param({"hls-audio-select": ["en"]}, "http://mocked/path/en.m3u8", id="English"),
-            pytest.param({"hls-audio-select": ["es"]}, "http://mocked/path/es.m3u8", id="Spanish"),
+            pytest.param({"hls-audio-select": ["EN"]}, "http://mocked/path/en.m3u8", id="English-alpha2"),
+            pytest.param({"hls-audio-select": ["ENG"]}, "http://mocked/path/en.m3u8", id="English-alpha3"),
+            pytest.param({"hls-audio-select": ["English"]}, "http://mocked/path/en.m3u8", id="English-name"),
+            pytest.param({"hls-audio-select": ["English Language"]}, "http://mocked/path/en.m3u8", id="English-name-attr"),
+            pytest.param({"hls-audio-select": ["es"]}, "http://mocked/path/es.m3u8", id="Spanish-alpha2"),
+            pytest.param({"hls-audio-select": ["spa"]}, "http://mocked/path/es.m3u8", id="Spanish-alpha3"),
+            pytest.param({"hls-audio-select": ["spanish"]}, "http://mocked/path/es.m3u8", id="Spanish-name"),
+            pytest.param({"hls-audio-select": ["spanish language"]}, "http://mocked/path/es.m3u8", id="Spanish-name-attr"),
         ],
         indirect=["session"],
     )
@@ -1131,8 +1171,10 @@ class TestHlsExtAudio:
     @pytest.mark.parametrize(
         "session",
         [
-            pytest.param({"hls-audio-select": ["*"]}, id="wildcard"),
-            pytest.param({"hls-audio-select": ["en", "es"]}, id="multiple locales"),
+            pytest.param({"hls-audio-select": ["en", "es"]}, id="alpha2"),
+            pytest.param({"hls-audio-select": ["eng", "spa"]}, id="alpha3"),
+            pytest.param({"hls-audio-select": ["English", "Spanish"]}, id="name"),
+            pytest.param({"hls-audio-select": ["English language", "Spanish language"]}, id="name-attr"),
         ],
         indirect=["session"],
     )
@@ -1144,6 +1186,54 @@ class TestHlsExtAudio:
             "http://mocked/path/es.m3u8",
         ]
 
+    @pytest.mark.parametrize(
+        "session",
+        [
+            pytest.param({"hls-audio-select": ["*"]}, id="wildcard"),
+        ],
+        indirect=["session"],
+    )
+    def test_wildcard(self, session: Streamlink, stream: MuxedHLSStream):
+        assert isinstance(stream, MuxedHLSStream)
+        assert [substream.url for substream in stream.substreams] == [
+            "http://mocked/path/playlist.m3u8",
+            "http://mocked/path/en.m3u8",
+            "http://mocked/path/es.m3u8",
+            "http://mocked/path/de.m3u8",
+        ]
+
+    @pytest.mark.parametrize(
+        ("session", "_playlist"),
+        [
+            pytest.param(
+                {"hls-audio-select": ["und", "qaa", "invalid"]},
+                {"playlist": "hls/test_media_language_special.m3u8"},
+                id="special-reserved-invalid",
+            ),
+            pytest.param(
+                {"hls-audio-select": ["Undetermined", "Reserved", "does NOT exist"]},
+                {"playlist": "hls/test_media_language_special.m3u8"},
+                id="name-attribute",
+            ),
+        ],
+        indirect=["session", "_playlist"],
+    )
+    def test_parse_media_language(self, caplog: pytest.LogCaptureFixture, session: Streamlink, stream: MuxedHLSStream):
+        assert isinstance(stream, MuxedHLSStream)
+        assert [substream.url for substream in stream.substreams] == [
+            "http://mocked/path/playlist.m3u8",
+            "http://mocked/path/qaa.m3u8",
+            "http://mocked/path/invalid.m3u8",
+            "http://mocked/path/und.m3u8",
+        ]
+        assert [
+            record.message
+            for record in caplog.get_records(when="setup")
+            if record.name == "streamlink.stream.hls" and record.levelno == logging.WARNING
+        ] == [
+            "Unrecognized language for media playlist: language='invalid' name='Does not exist'",
+        ]
+
 
 class TestM3U8ParserLogging:
     @pytest.mark.parametrize(("loglevel", "has_logs"), [("trace", False), ("all", True)])
